#include <Servo.h>
#include "constants.h"
#include "io_pins.h"


// define global variables

int wander=0;
int Fscandir=10;            // determines the scan speed of the front sensor servo (direction changes automatically)

int lp=5;                  // left  pan  value (scanning)
int rp=5;                  // right pan  value (scanning)
int lt=-1;                 // left  tilt value (scanning)
int rt=+1;                 // right tilt value (scanning)

int Ldistance;             // average reading for left  eye
int Rdistance;             // average reading for right eye

int LLIRvalue;             // left  eye - left  IR reflected value
int LRIRvalue;             // left  eye - right IR reflected value
int LUIRvalue;             // left  eye - up    IR reflected value
int LDIRvalue;             // left  eye - down  IR reflected value

int RLIRvalue;             // right eye - left  IR reflected value
int RRIRvalue;             // right eye - right IR reflected value
int RUIRvalue;             // right eye - up    IR reflected value
int RDIRvalue;             // right eye - down  IR reflected value
 
int drumbeat;              // position in drum sequence
int stride=25;             // position in walking sequence 
int pos=0;                 // 0 = Sitting    28 = Standing    56 = Curled Up    84 = standing tall

int Lspeed;                // determines left  legs stride and direction (-4 to +4)
int Rspeed;                // determines right legs stride and direction (-4 to +4)
int Cspeed;                // determines crab walk  stride and direction (-4 to +4) 
int Aspeed;                // average speed used to adjust walkspeed

int OLspeed;               // Old left  speed value 
int ORspeed;               // Old right speed value 
int OCspeed;               // Old crab  speed value (straffing speed)
int lift;                  // amount to lift legs by when walking, drumming etc.

int patience=3000;         // The level of patience will depend on emotional factors: 1000 = aproximately 10sec.
int track=-500;            // track is a counter that determines how much stimulus the robot gets from tracking objects.

int walkspeed=25;          // number of milliseconds between walk sequence steps
int drumspeed=100;         // set the tempo of the drum sequence
unsigned long drumtime;    // timer used to control drum tempo
unsigned long walktime;    // timer used to control walking speed
unsigned long modetime;    // used to prevent the robot changing between walkmodes too quickly


// define servo center positions (Spider at rest)
int svc[]={
  1900,1850,1950,1850,     // D30 thigh4, D31 thigh3, D32 thigh2, D33 thigh1
  1500,1500,1500,1500,     // D34 knee 4, D35 knee 3, D36 knee 2, D37 knee 1
  1580,1650,1600,1850,     // D38 hip  4, D39 hip  3, D40 hip  2, D41 hip  1
  1350,1535,1580,1790,     // D42 hip  8, D43 hip  7, D44 hip  6, D45 hip  5
  1500,1500,1500,1500,     // D46 knee 8, D47 knee 7, D48 knee 6, D49 knee 5
  1150,1050,1150,1200,     // D50 thigh8, D51 thigh7, D52 thigh6, D53 thigh5 
  0,   0,   0,   0,        // D26 Lpan  , D27 Ltilt , D28 Rpan  , D29 Rtilt

  // define servo center positions (Spider standing)
  1350,1300,1500,1300,     // D30 thigh4, D31 thigh3, D32 thigh2, D33 thigh1
  1550,1550,1550,1600,     // D34 knee 4, D35 knee 3, D36 knee 2, D37 knee 1
  1480,1550,1600,1850,     // D38 hip  4, D39 hip  3, D40 hip  2, D41 hip  1
  1350,1600,1800,1950,     // D42 hip  8, D43 hip  7, D44 hip  6, D45 hip  5
  1400,1450,1400,1450,     // D46 knee 8, D47 knee 7, D48 knee 6, D49 knee 5
  1700,1700,1750,1750,     // D50 thigh8, D51 thigh7, D52 thigh6, D53 thigh5 
  0,   0,   0,   0,        // D26 Lpan  , D27 Ltilt , D28 Rpan  , D29 Rtilt
  
   // define servo center positions (Spider curled up)
  2350,2300,2350,2300,     // D30 thigh4, D31 thigh3, D32 thigh2, D33 thigh1
  2200,2200,2200,2200,     // D34 knee 4, D35 knee 3, D36 knee 2, D37 knee 1
  1680,1650,1500,1650,     // D38 hip  4, D39 hip  3, D40 hip  2, D41 hip  1
  1550,1635,1580,1690,     // D42 hip  8, D43 hip  7, D44 hip  6, D45 hip  5
   800, 800, 800, 800,     // D46 knee 8, D47 knee 7, D48 knee 6, D49 knee 5
   750, 600, 750, 750,     // D50 thigh8, D51 thigh7, D52 thigh6, D53 thigh5 
  2100,2000,2100,2000,     // D26 Lpan  , D27 Ltilt , D28 Rpan  , D29 Rtilt

  // define servo center positions (Spider standing tall)
   800, 750, 850, 750,     // D30 thigh4, D31 thigh3, D32 thigh2, D33 thigh1
  1100,1100,1150,1100,     // D34 knee 4, D35 knee 3, D36 knee 2, D37 knee 1
  1580,1650,1600,1850,     // D38 hip  4, D39 hip  3, D40 hip  2, D41 hip  1
  1350,1535,1580,1790,     // D42 hip  8, D43 hip  7, D44 hip  6, D45 hip  5
  1800,1800,1750,1800,     // D46 knee 8, D47 knee 7, D48 knee 6, D49 knee 5
  2200,2250,2250,2250,     // D50 thigh8, D51 thigh7, D52 thigh6, D53 thigh5
  0,   0,   0,   0         // D26 Lpan  , D27 Ltilt , D28 Rpan  , D29 Rtilt
};


int svt[29];               // storage of temporary servo calculations used to control speed and direction of servos

int svp[29];               // initial servo position (center)

//----------------------------------------------------------- define servos ---------------------------------------------------------

Servo sv[29];                                              // Yes! servos can be an array

void setup()
{
  for(int i=0;i<28;i++)                                    // copy center positions into servo position array
  {
    svp[i]=svc[i];
    if (svp[i]==0) svp[i]=1500;     
  }
  
  svp[28]=1500;
  delay(1000);
  pinMode(Srvpwrpin,OUTPUT);                               // pin used to turn on/off servo power
  digitalWrite(Srvpwrpin,1);                               // power up servos - servos remain dead until they are initialized and receive a signal

  //--------------------------------------------------------- Servos initialized in a sequence to prevent the legs becoming tangled ---

  sv[24].attach(Lpanpin,Lpanmin,Lpanmax);                  // initialize left eye pan servo
  sv[24].writeMicroseconds(Lpanc);
  sv[25].attach(Ltiltpin,Ltiltmin,Ltiltmax);               // initialize left eye tilt servo
  sv[25].writeMicroseconds(Ltiltc);
  sv[26].attach(Rpanpin,Rpanmin,Rpanmax);                  // initialize right eye pan servo
  sv[26].writeMicroseconds(Rpanc);
  sv[27].attach(Rtiltpin,Rtiltmin,Rtiltmax);               // initialize right eye tilt servo
  sv[27].writeMicroseconds(Rtiltc);
  
  sv[28].attach(Fsenspin,Fsensmin,Fsensmax);               // initialize front sensor scan servo
  sv[28].writeMicroseconds(Fsensc);


  for(int i=0;i<4;i++)                                     // initialize left thighs
  {
    sv[i].attach(legpins+i,700,2300); 
    sv[i].writeMicroseconds(svc[i]);
    delay(50);
  }

  for(int i=20;i<24;i++)                                   // initialize right thighs
  {
    sv[i].attach(legpins+i,700,2300); 
    sv[i].writeMicroseconds(svc[i]);
    delay(50);
  }

  for(int i=8;i<16;i++)                                    // initialize all hips
  {
    sv[i].attach(legpins+i,700,2300); 
    sv[i].writeMicroseconds(svc[i]);
    delay(50);
  }

  for(int i=4;i<8;i++)                                     // initialize left knees
  {
    sv[i].attach(legpins+i,600,2500); 
    sv[i].writeMicroseconds(svc[i]);
    delay(50);
  }

  for(int i=16;i<20;i++)                                   // initialize right knees
  {
    sv[i].attach(legpins+i,600,2500); 
    sv[i].writeMicroseconds(svc[i]);
    delay(50);
  }

  delay(1000);
  if (pos>0) SitStand();                                   // once initialized, the spider is in a sitting position

  Serial.begin(57600);
  walktime=millis();
  delay(1000);
}


void loop()
{
  IReyes();                                                // read information from eyes
  IRtrackobject();                                         // pan/tilt eyes to track objects
  Serial.print("\tLeft Pan:");Serial.print(svp[24]);
  Serial.print("\tLeft Tilt:");Serial.print(svp[25]);
  Serial.print("\tRight Pan:");Serial.print(svp[26]);
  Serial.print("\tRight Tilt:");Serial.print(svp[27]);
  Serial.print("\tScan:");Serial.println(svp[28]);
  delay(10);
 
  
  svp[28]+=Fscandir;
  if (svp[28]>Fscanmax || svp[28]<Fscanmin) Fscandir=-Fscandir;
  sv[28].writeMicroseconds(svp[28]);
   return;
  
  if (Ldistance>tooclose && Rdistance>tooclose)            // object too close, hide!
  {
    walkspeed=25;
    HideEyes();
    delay(1000);
    Peek();                                                // look to see if it is safe
    track=patience;                                        // not bored - reset counter
    wander=0;
  }

  if (track>25000)
  {
    wander=0;
    track=0;
  }
  
  track-=1;
  if(track>patience) track=patience;
  if(track<-patience/2)
  {
    track=0;
    wander++;
    if (wander>5) track=20000;
  }
  
  if(track<0 && wander<10)//---------------------------------------------- the robot is bored -----------------------------------------------------------------
  {
    if(pos==28)                                            // if the robot is standing then sit down slowly
    {
      pos=0;
      walkspeed=50;
      lp=1;                                                // set left eye scan to lazy
      rp=1;                                                // set right eye scan to lazy
      SitStand();
    }
    if ((millis()-drumtime)>drumspeed)                     // control the tempo of the drumbeat
    {
      drumtime=millis();
      Drum();                                              // drumbeat sequence
    }
  }

  if (track>100)//------------------------------------------- robot is not bored ------------------------------------------------------------------
  {
    if (pos!=28)                                           // get back on feet if required
    {
      pos=28;
      walkspeed=25;
      lp=5;                                                // reset left eye scan to active
      rp=5;                                                // reset right eye scan to active
      SitStand();
    }
    FollowObject();
  }
}





//=========================================================== Brings the robot to a predefined position =============================

void SitStand()                                                                                     
{
  for(int i=0;i<28;i++)                                    // scan through servo positions
  {
    svt[i]=(svc[i+pos]-svp[i])/20;                         // calculate speed and direction required to move the servo to new position in 20 steps
    if(svc[i+pos]==0) svt[i]=0;                            // a 0 indicates do not change servo position

  }
  for(int j=0;j<20;j++)                                    // move servos over a period of 20 steps to generate a smooth motion
  {
    for(int i=0;i<28;i++)
    {
      svp[i]+=svt[i];                                      // adjust the servos position
      sv[i].writeMicroseconds(svp[i]);                     // update the servos position
    }
    delay(walkspeed);                                      // controls speed of motion
  }
}



//=========================================================== Update leg servo positions ============================================

void ServoUpdate()
{
  for(int i=0;i<24;i++)                                    // update leg servo positions
  {
    sv[i].writeMicroseconds(svp[i]);
  }
}




















